/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.*;
import javax.vecmath.Point3d;

import org.jet.sce.*;

/**
 * <i>BasePoint</i> dialog.
 *
 * @author Adam Domaski
 * @version 0.1 25/02/2004
 */
public class BasePointDialog extends JetDialog
{
	/**
	 * Creates dialog.
	 *
	 * @param frame Parent frame.
	 * @param editor Editor.
	 */
	public BasePointDialog(Frame frame, JSceneryEditor editor)
	{
		super(frame);

		resource = JetResources.getInstance();
		setTitle( resource.translate("BasePointDialog.title") );

		setResizable(false);
		setDefaultCloseOperation(HIDE_ON_CLOSE);

		this.editor = editor;

		initialize();
		fillIn();

		pack();
		centerDialog();
		setVisible(true);
	}

	private void initialize()
	{
		JPanel jPane = new JPanel();
		jPane.setLayout(new BorderLayout());

		JPanel jContainer = createContents();
		jPane.add(jContainer);

		JPanel jControlPanel = new JPanel();
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		jControlPanel.setLayout(gb);
		jControlPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		JButton okButton = new JButton( resource.translate("BasePointDialog.ok") );
		JButton caButton = new JButton( resource.translate("BasePointDialog.cancel") );

		caButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae)
			{
				dispose();
			}
		});

		okButton.addActionListener(new OkAction());
		getRootPane().setDefaultButton(okButton);

		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.EAST;
		c.insets = new Insets(0, 0, 0, 5);
		gb.setConstraints(okButton, c);
		jControlPanel.add(okButton);

		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 0.0;
		c.insets = new Insets(0,0,0,0);
		gb.setConstraints(caButton, c);
		jControlPanel.add(caButton);

		normalize(okButton, caButton);
		okButton.requestFocus();

		jPane.add(jControlPanel, BorderLayout.SOUTH);

		getContentPane().add(jPane);
	}

	/**
	 * Creates contents.
	 */
	private JPanel createContents()
	{
		JLabel jLabel;
		JFormattedTextField jText;

		JPanel jMainPane = new JPanel();
		jMainPane.setName("GoToDialogPanel");
		jMainPane.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();		
		Insets insets = new Insets(1, 2, 1, 2);
		c.insets = insets;
		jMainPane.setLayout(gb);

		JPanel jPane = new JPanel();
		jPane.setLayout(new GridLayout(3, 2, 0, 2));
		jPane.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );

		jLabel = new JLabel();
		jLabel.setName("XTextBoxLabel");
		jLabel.setText( resource.translate("BasePointDialog.x") );
		jPane.add(jLabel);

		jText = JetDialog.createLongFormatTextBox();
		jText.setName("XTextBox");
		jText.setColumns(6);
		controls.putComponent(jText);
		jPane.add(jText);

		jLabel = new JLabel();
		jLabel.setName("YTextBoxLabel");
		jLabel.setText( resource.translate("BasePointDialog.y") );
		jPane.add(jLabel);

		jText = JetDialog.createLongFormatTextBox();
		jText.setName("YTextBox");
		jText.setColumns(6);
		controls.putComponent(jText);
		jPane.add(jText);

		jLabel = new JLabel();
		jLabel.setName("ZTextBoxLabel");
		jLabel.setText( resource.translate("BasePointDialog.z") );
		jPane.add(jLabel);

		jText = JetDialog.createLongFormatTextBox();
		jText.setName("ZTextBox");
		jText.setColumns(6);
		controls.putComponent(jText);
		jPane.add(jText);

		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jPane, c);
		jMainPane.add(jPane);

		jPane = new JPanel();
		jPane.setLayout(new GridLayout(2, 2, 0, 2));
		jPane.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );

		jLabel = new JLabel();
		jLabel.setName("AlphaLabel");
		jLabel.setText( resource.translate("BasePointDialog.alpha") );
		jPane.add(jLabel);

		jText = JetDialog.createDegreeFormatTextBox();
		jText.setName("AlphaTextBox");
		jText.setColumns(6);
		controls.putComponent(jText);
		jPane.add(jText);

		jLabel = new JLabel();
		jLabel.setName("BetaLabel");
		jLabel.setText( resource.translate("BasePointDialog.beta") );
		jPane.add(jLabel);

		jText = JetDialog.createDegreeFormatTextBox();
		jText.setName("BetaTextBox");
		jText.setColumns(6);
		controls.putComponent(jText);
		jPane.add(jText);

		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jPane, c);
		jMainPane.add(jPane);		

		return jMainPane;
	}

	private void fillIn()
	{
		JFormattedTextField jText;
		BasePoint bp = editor.getBasePoint();
		if (bp != null)
		{
			jText = (JFormattedTextField) controls.getComponent("XTextBox");
			jText.setValue( new Double(bp.getPoint().x) );
			jText = (JFormattedTextField) controls.getComponent("YTextBox");
			jText.setValue( new Double(bp.getPoint().y) );
			jText = (JFormattedTextField) controls.getComponent("ZTextBox");
			jText.setValue( new Double(bp.getPoint().z) );
			jText = (JFormattedTextField) controls.getComponent("AlphaTextBox");
			jText.setValue( new Double(Math.toDegrees(bp.getAlpha())) );
			jText = (JFormattedTextField) controls.getComponent("BetaTextBox");
			jText.setValue( new Double(Math.toDegrees(bp.getBeta())) );
		}
		else
		{
			jText = (JFormattedTextField) controls.getComponent("XTextBox");
			jText.setValue( new Double(0.0) );
			jText = (JFormattedTextField) controls.getComponent("YTextBox");
			jText.setValue( new Double(0.0) );
			jText = (JFormattedTextField) controls.getComponent("ZTextBox");
			jText.setValue( new Double(0.0) );
			jText = (JFormattedTextField) controls.getComponent("AlphaTextBox");
			jText.setValue( new Double(0.0) );
			jText = (JFormattedTextField) controls.getComponent("BetaTextBox");
			jText.setValue( new Double(0.0) );
		}

		setConstraints();
	}

	private void setConstraints()
	{
		JFormattedTextField jFormatText;
		
		jFormatText = (JFormattedTextField) controls.getComponent("BetaTextBox");
		setConstraints(jFormatText, new Double(-90.0), new Double(90.0));
	}

	/**
	 * "OK" Action.
	 */
	class OkAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			double alpha, beta;
			JFormattedTextField jFormat;

			Point3d p3d = new Point3d();
			jFormat = (JFormattedTextField) controls.getComponent("XTextBox");
			p3d.x = ((Number) jFormat.getValue()).doubleValue();
			jFormat = (JFormattedTextField) controls.getComponent("YTextBox");
			p3d.y = ((Number) jFormat.getValue()).doubleValue();
			jFormat = (JFormattedTextField) controls.getComponent("ZTextBox");
			p3d.z = ((Number) jFormat.getValue()).doubleValue();
			jFormat = (JFormattedTextField) controls.getComponent("AlphaTextBox");
			alpha = Math.toRadians( ((Number) jFormat.getValue()).doubleValue() );
			jFormat = (JFormattedTextField) controls.getComponent("BetaTextBox");
			beta = Math.toRadians( ((Number) jFormat.getValue()).doubleValue() );
			BasePoint bp = new BasePoint(p3d, alpha, beta);

			editor.setBasePoint(bp);

			editor.centerViewAt( editor.modelToView( bp.getPoint()) );

			dispose();
		}
	}

	/*
	 * --------------- Members ---------------
	 */
	
	/** I18n resources. */
	private JetResources resource;

	/** Components. */
	private ControlMap controls = new ControlMap();

	/** Scenery editor */
	private JSceneryEditor editor;
}
