/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package org.jet;

import java.util.ResourceBundle;
import java.util.MissingResourceException;

/**
 * A source of locale text manipulation.
 * <p>To get an instance of this class use static mathod <code>getInstance</code>.</p>
 *
 * @author Adam Domaski
 * @version 0.1 11/02/2004
 */
public class JetResources
{
	private static JetResources instance = null;
	
	private ResourceBundle resource;

	private JetResources()
	{
		try
		{
			resource = ResourceBundle.getBundle("i18n/jet");
		}
		catch (MissingResourceException mre)
		{
			mre.printStackTrace();
		}
	}

	/**
	 * Gets the instance of this class.
	 *
	 * @return Jet resource.
	 */
	public static JetResources getInstance()
	{
		if (instance == null)
		{
			instance = new JetResources();
		}

		return instance;
	}

	/**
	 * Translates given string with current (default) locale.
	 * <p>Locale dependent data is stored in <code>i18n</code> directory.</p>
	 *
	 * @param str String to translate.
	 * @return Translated string.
	 */
	public String translate(String str)
	{
		String ret = null;
		try
		{
			ret = resource.getString(str);
		}
		catch (MissingResourceException mse)
		{
			mse.printStackTrace();
		}
		return ret;
	}
}
