/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.*;

import org.jet.sce.*;

/**
 * Scenery description dialog.
 *
 * @author Adam Domaski
 * @version 0.1 16/02/2004
 */
public class SceneryDescriptionDialog extends JetDialog
{
	/**
	 * Creates scenery description dialog within frame and
	 *   with scenery frame.
	 *
	 * @param frame Parent window.
	 * @param scf Scenery frame where description is stored.
	 */
	public SceneryDescriptionDialog(Frame frame, SceneryFrame scf)
	{
		super(frame);

		resource = JetResources.getInstance();
		setTitle( resource.translate("SceneryDescriptionDialog.title") );

		setResizable(false);
		setDefaultCloseOperation(HIDE_ON_CLOSE);

		okAction = new OKAction( scf.getSceneryEditor().getScenery() );

		initialize();
		fillIn(scf);

		pack();
		centerDialog();
		setVisible(true);
	}

	private void initialize()
	{
		JPanel jPane = new JPanel();
		jPane.setName("SceneryDescriptionPanel");
		jPane.setLayout(new BorderLayout());

		JPanel jContainer = createContents();
		jPane.add(jContainer);

		JPanel jControlPanel = new JPanel();
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		jControlPanel.setLayout(gb);
		jControlPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		JButton okButton = new JButton( resource.translate("SceneryDescriptionDialog.ok") );
		JButton caButton = new JButton( resource.translate("SceneryDescriptionDialog.cancel") );

		caButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae)
			{
				dispose();
			}
		});

		okButton.addActionListener(okAction);
		getRootPane().setDefaultButton(okButton);

		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.EAST;
		c.insets = new Insets(0, 0, 0, 5);
		gb.setConstraints(okButton, c);
		jControlPanel.add(okButton);

		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 0.0;
		c.insets = new Insets(0,0,0,0);
		gb.setConstraints(caButton, c);
		jControlPanel.add(caButton);

		normalize(okButton, caButton);
		okButton.requestFocus();

		jPane.add(jControlPanel, BorderLayout.SOUTH);
		
		getContentPane().add(jPane);
	}

	private JPanel createContents()
	{
		JLabel jLabel;
		JTextArea jTextArea;

		JPanel jPane = new JPanel();
		jPane.setBorder( BorderFactory.createEmptyBorder(5, 5, 5, 5) );
		BoxLayout boxLayout = new BoxLayout(jPane, BoxLayout.PAGE_AXIS);
		jPane.setLayout(boxLayout);

		jLabel = new JLabel();
		jLabel.setName("DescriptionLabel");
		jLabel.setText( resource.translate("SceneryDescriptionDialog.description") );
		jPane.add(jLabel);

		jPane.add( Box.createRigidArea(new Dimension(0, 3)) );

		jTextArea = new JTextArea();
		jTextArea.setName("SceneryDescriptionArea");
		jTextArea.setLineWrap(true);
		jTextArea.setWrapStyleWord(true);
		controls.putComponent(jTextArea);
		JScrollPane jScrollPane = new JScrollPane(jTextArea);
		jScrollPane.setPreferredSize(new Dimension(300, 200));
		jScrollPane.setAlignmentX(JComponent.LEFT_ALIGNMENT);
		jPane.add(jScrollPane);

		return jPane;
	}

	private void fillIn(SceneryFrame sf)
	{
		JTextArea jTextArea;
		Scenery scenery = sf.getSceneryEditor().getScenery();

		jTextArea = (JTextArea) controls.getComponent("SceneryDescriptionArea");
		jTextArea.append( scenery.getDescription() );
	}

	class OKAction implements ActionListener
	{
		public OKAction(Scenery scenery)
		{
			this.scenery = scenery;
		}

		public void actionPerformed(ActionEvent ae)
		{
			JTextArea jTextArea;
			
			jTextArea = (JTextArea) controls.getComponent("SceneryDescriptionArea");
			scenery.setDescription( jTextArea.getText() );
			dispose();
		}

		private Scenery scenery;
	}

	/*
	 * ---------------- Members -------------------
	 */

	private ControlMap controls = new ControlMap();

	private OKAction okAction;

	private JetResources resource;
}
