/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import javax.swing.ComboBoxEditor;
import javax.swing.JFormattedTextField;
import java.awt.Component;
import java.awt.event.ActionListener;

/**
 * Formatted text field as the combo box editor.
 *
 * @author Adam Domaski
 * @version 0.1 22/08/2003
 */
public class ZoomComboBoxEditor implements ComboBoxEditor
{
	/**
	 * Creates new editor for combo box.
	 * An instance of this editor is a formatted text field.
	 */
	public ZoomComboBoxEditor()
	{
		editor = new JFormattedTextField();
	}

	/**
	 * Gets the GUI component to be used as editor.
	 */
	public Component getEditorComponent()
	{
		return editor;
	}

	/**
	 * Gets edited value.
	 */
	public Object getItem()
	{
		return editor.getValue();
	}

	/**
	 * Sets editor with value.
	 */
	public void setItem(Object item)
	{
		editor.setValue(item);
	}

	/**
	 * Selects content of the editor.
	 */
	public void selectAll()
	{
		editor.selectAll();
	}

	/**
	 * Adds action listener to the editor.
	 */
	public void addActionListener(ActionListener listener)
	{
		editor.addActionListener(listener);
	}

	/**
	 * Removes action listener from the editor.
	 */
	public void removeActionListener(ActionListener listener)
	{
		editor.removeActionListener(listener);
	}

	/**
	 * Gets editor.
	 */
	public JFormattedTextField getTextField()
	{
		return editor;
	}

	/*
	 * ----------- Members -------------
	 */

	/** Editor component. */
	protected JFormattedTextField editor;
}
