/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.sce;

import java.io.*;
import java.util.Iterator;

import prv.doman.fsa.*;
import prv.doman.util.*;
import org.jet.tracks.*;

/**
 * Scenery editor kit is a place where some useful utilities are stored.
 *
 * @author Adam Domaski
 * @version 0.1 12/03/2004
 */
public class EditorKit
{
	/*
	 * ----------- reading / writing -------------
	 */
	
	/**
	 * Reads input stream which is expected to be a
	 * scenery.
	 *
	 * @param in The stream to read.
	 * @param sc The scenery to write to.
	 * @exception IOException When some errors occured.
	 */
	public void read(InputStream in, Scenery sc) throws IOException
	{
		read(new InputStreamReader(in), sc);
	}

	/**
	 * Reads reader which is expected to be content for scenery.
	 *
	 * @param rin The reader to read.
	 * @param sc The scenery to write to.
	 * @exception IOException When some errors occured.
	 */
	/*
	 * --------- Need to be optimized. -----------
	 * Bottlenacks: tokenizer
	 */
	public void read(Reader rin, Scenery sc) throws IOException
	{
		String word = null;
		boolean wasDescription = false;
		StringBuffer desc = new StringBuffer();

		String nodeName = null;
		Track track = null;
		Switch sswitch = null;
		double[] tupleBuffer = new double[3];

		Automaton automat = new Automaton();
		initAutomaton(automat);

		Tokenizer tokenizer = new Tokenizer(new BufferedReader(rin));
		while (tokenizer.hasMoreTokens())
		{
			Token token = tokenizer.nextToken();

			int ttype = token.getType();
			switch (ttype)
			{
			case Token.TOKEN_EOL:
				word = "eol";
				break;
			case Token.TOKEN_WORD:
				word = getWord( token.getWord() );
				break;
			case Token.TOKEN_NUMBER:
				word = "number";
				break;
			}

			// Wykonaj dodatkowe operacje nad sowami automatu
			if (word.equals("enddescription"))
				wasDescription = true;

			// Wykonaj ruch w maszynie
			String state = automat.currentState();
			if (state.equals("description") && !wasDescription)
			{
				if (ttype == Token.TOKEN_EOL)
				{
					desc.append('\n');
				}
				else
				{
					desc.append(token.getWord());
					desc.append(' ');
				}
			}
			else if (word.equals("endtrack") && (state.equals("track event")
				|| state.equals("track geom radius")))
			{
				if (ttype == Token.TOKEN_WORD)
				{
					Track[] t = new Track[1];
					t[0] = track;
					sc.insert(t);
				}
			}
			else if (word.equals("endtrack") && (state.equals("switch event")
				|| state.equals("switch rad 2")))
			{
				if (ttype == Token.TOKEN_WORD)
				{
					Switch[] s = new Switch[1];
					s[0] = sswitch;
					sc.insert(s);
				}
			}


			// Wykonaj ruch
			try
			{
				automat.step(word);
			}
			catch (AutomatonException ae) {}

			// Czynnoci po ruchu
			state = automat.currentState();
			if (state.equals("error"))
			{
				rin.close();
				//System.out.println(token.getColumn() + " " + word + " " + token.getWord());
				throw new SceneryException(
					"B\u0142\u0105d sk\u0142adni pliku scenerii.",
					tokenizer.getLine());
			}
			else if (state.equals("state4") && !token.getWord().equals("none"))
			{
				if (ttype == Token.TOKEN_WORD)
				{
					nodeName = token.getWord();
				}
			}
			else if (state.equals("inside track 1"))
			{
				track = new Track();
				track.setName(nodeName);
				nodeName = null;
			}
			else if (state.equals("track geom p1 1") || state.equals("track geom p2 1") ||
				state.equals("track geom v1 1") || state.equals("track geom v2 1"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[0] = ((NumberToken) token).getNumber().doubleValue();
				}
			}
			else if (state.equals("track geom p1 2") || state.equals("track geom p2 2") ||
				state.equals("track geom v1 2") || state.equals("track geom v2 2"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[1] = ((NumberToken) token).getNumber().doubleValue();
				}
			}
			else if (state.equals("track geom p1 3"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[2] = ((NumberToken) token).getNumber().doubleValue();
					track.setPoint1(tupleBuffer);
				}
			}
			else if (state.equals("track geom p2 3"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[2] = ((NumberToken) token).getNumber().doubleValue();
					track.setPoint2(tupleBuffer);
				}
			}
			else if (state.equals("track geom v1 3"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[2] = ((NumberToken) token).getNumber().doubleValue();
					track.setVector1(tupleBuffer);
				}
			}
			else if (state.equals("track geom v2 3"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[2] = ((NumberToken) token).getNumber().doubleValue();
					track.setVector2(tupleBuffer);
				}
			}
			else if (state.equals("inside track 2"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					track.setLength( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("inside track 3"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					track.setWidth( ((NumberToken) token).getNumber().doubleValue() );
				}				
			}
			else if (state.equals("inside track 3a"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					track.setFriction( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("inside track 4"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					track.setTap( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("inside track 5"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					track.setQuality( ((NumberToken) token).getNumber().intValue() );
				}
			}
			else if (state.equals("inside track 6"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					track.setDamage( ((NumberToken) token).getNumber().intValue() );
				}
			}
			else if (state.equals("inside track 7"))
			{
				track.setEnviroment( token.getWord() );
			}
			else if (state.equals("inside track 8") || state.equals("inside track 9"))
			{
				if (word.equals("vis"))
				{
					track.setVisible(true);
				}
				else if (word.equals("unvis"))
				{
					track.setVisible(false);
				}
			}
			else if (state.equals("track tex 1"))
			{
				track.setTexture( token.getWord() );
			}
			else if (state.equals("track tex 2"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					track.setTextureLength( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("track tex 3"))
			{
				track.setSecondTexture( token.getWord() );
			}
			else if (state.equals("track tex 4"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					track.setTextureHeight( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("track tex 5"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					track.setTextureWidth( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("track tex 6"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					track.setTextureSlope( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("track geom radius"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					track.setRadius( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("track geom r1"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					track.setRoll1( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("track geom r2"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					track.setRoll2( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch"))
			{
				sswitch = new Switch();
				sswitch.setName(nodeName);
				nodeName = null;
			}
			else if (state.equals("switch length"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setLength( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch width"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setWidth( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch friction"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setFriction( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch tap"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setTap( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch quality"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setQuality( ((NumberToken) token).getNumber().intValue() );
				}
			}
			else if (state.equals("switch damage"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setDamage( ((NumberToken) token).getNumber().intValue() );
				}
			}
			else if (state.equals("switch env"))
			{
				sswitch.setEnviroment( token.getWord() );
			}
			else if (state.equals("switch vis") || state.equals("switch unvis"))
			{
				if (word.equals("vis"))
				{
					sswitch.setVisible(true);
				}
				else if (word.equals("unvis"))
				{
					sswitch.setVisible(false);
				}
			}
			else if (state.equals("switch tex 1"))
			{
				sswitch.setTexture( token.getWord() );
			}
			else if (state.equals("switch tex 2"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setTextureLength( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch tex 3"))
			{
				sswitch.setSecondTexture( token.getWord() );
			}
			else if (state.equals("switch tex 4"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setTextureWidth( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch tex 5"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setTextureHeight( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch tex 6"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setTextureSlope( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch geom p1 x") || state.equals("switch geom v1 x") ||
				state.equals("switch geom v2 x") || state.equals("switch geom p2 x") ||
				state.equals("switch geom p3 x") || state.equals("switch geom v3 x") ||
				state.equals("switch geom v4 x") || state.equals("switch geom p4 x"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[0] = ((NumberToken) token).getNumber().doubleValue();
				}
			}
			else if (state.equals("switch geom p1 y") || state.equals("switch geom v1 y") ||
				state.equals("switch geom v2 y") || state.equals("switch geom p2 y") ||
				state.equals("switch geom p3 y") || state.equals("switch geom v3 y") ||
				state.equals("switch geom v4 y") || state.equals("switch geom p4 y"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[1] = ((NumberToken) token).getNumber().doubleValue();
				}
			}
			else if (state.equals("switch geom p1 z"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[2] = ((NumberToken) token).getNumber().doubleValue();
					sswitch.setPoint1( tupleBuffer );
				}
			}
			else if (state.equals("switch geom p2 z"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[2] = ((NumberToken) token).getNumber().doubleValue();
					sswitch.setPoint2( tupleBuffer );
				}
			}
			else if (state.equals("switch geom p3 z"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[2] = ((NumberToken) token).getNumber().doubleValue();
				}
			}
			else if (state.equals("switch geom p4 z"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[2] = ((NumberToken) token).getNumber().doubleValue();
					sswitch.setPoint3( tupleBuffer );
				}
			}
			else if (state.equals("switch geom v1 z"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[2] = ((NumberToken) token).getNumber().doubleValue();
					sswitch.setVector1( tupleBuffer );
				}
			}
			else if (state.equals("switch geom v2 z"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[2] = ((NumberToken) token).getNumber().doubleValue();
					sswitch.setVector2( tupleBuffer );
				}
			}
			else if (state.equals("switch geom v3 z"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[2] = ((NumberToken) token).getNumber().doubleValue();
					sswitch.setSwitchedVector( tupleBuffer );
				}
			}
			else if (state.equals("switch geom v4 z"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					tupleBuffer[2] = ((NumberToken) token).getNumber().doubleValue();
					sswitch.setVector3( tupleBuffer );
				}
			}
			else if (state.equals("switch rad 1"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setRadius( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch rad 2"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setSwitchedRadius( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch geom p1 r"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setRoll1( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch geom p2 r"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setRoll2( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch geom p3 r"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setSwitchedRoll( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("switch geom p4 r"))
			{
				if (ttype == Token.TOKEN_NUMBER)
				{
					sswitch.setRoll3( ((NumberToken) token).getNumber().doubleValue() );
				}
			}
			else if (state.equals("state1"))
			{
				nodeName = null;
			}

		}

		// Dodaj opis scenerii
		removeNewLines(desc);
		sc.setDescription(desc.toString());
	}

	/**
	 * Initialized automaton for file reading.
	 *
	 * @param a Reference to created automaton.
	 */
	private void initAutomaton(Automaton a)
	{
		try
		{
			a.addLetter("inne");
			a.addLetter("node");
			a.addLetter("number");
			a.addLetter("track");
			a.addLetter("endtrack");
			a.addLetter("description");
			a.addLetter("enddescription");
			a.addLetter("eol");
			a.addLetter("normal");
			a.addLetter("river");
			a.addLetter("switch");
			a.addLetter("road");
			a.addLetter("vis");
			a.addLetter("unvis");

			a.addState("ignore");
			a.addState("state1");
			a.addState("state2");
			a.addState("state3");
			a.addState("state4");
			a.addState("track");
			a.addState("error");
			a.addState("description");
			a.addState("inside track 1");
			a.addState("inside track 2");
			a.addState("inside track 3");
			a.addState("inside track 3a");
			a.addState("inside track 4");
			a.addState("inside track 5");
			a.addState("inside track 6");
			a.addState("inside track 7");
			a.addState("inside track 8");
			a.addState("inside track 9");
			a.addState("track tex 1");
			a.addState("track tex 2");
			a.addState("track tex 3");
			a.addState("track tex 4");
			a.addState("track tex 5");
			a.addState("track tex 6");
			a.addState("track geom p1 1");
			a.addState("track geom p1 2");
			a.addState("track geom p1 3");
			a.addState("track geom r1");
			a.addState("track geom v1 1");
			a.addState("track geom v1 2");
			a.addState("track geom v1 3");
			a.addState("track geom v2 1");
			a.addState("track geom v2 2");
			a.addState("track geom v2 3");
			a.addState("track geom p2 1");
			a.addState("track geom p2 2");
			a.addState("track geom p2 3");
			a.addState("track geom r2");
			a.addState("track geom radius");
			a.addState("track event");
			a.addState("other track");
			a.addState("switch");
			a.addState("switch length");
			a.addState("switch width");
			a.addState("switch friction");
			a.addState("switch tap");
			a.addState("switch quality");
			a.addState("switch damage");
			a.addState("switch env");
			a.addState("switch vis");
			a.addState("switch unvis");
			a.addState("switch tex 1");
			a.addState("switch tex 2");
			a.addState("switch tex 3");
			a.addState("switch tex 4");
			a.addState("switch tex 5");
			a.addState("switch tex 6");
			a.addState("switch geom p1 x");
			a.addState("switch geom p1 y");
			a.addState("switch geom p1 z");
			a.addState("switch geom p1 r");
			a.addState("switch geom v1 x");
			a.addState("switch geom v1 y");
			a.addState("switch geom v1 z");
			a.addState("switch geom v2 x");
			a.addState("switch geom v2 y");
			a.addState("switch geom v2 z");
			a.addState("switch geom p2 x");
			a.addState("switch geom p2 y");
			a.addState("switch geom p2 z");
			a.addState("switch geom p2 r");
			a.addState("switch rad 1");
			a.addState("switch geom p3 x");
			a.addState("switch geom p3 y");
			a.addState("switch geom p3 z");
			a.addState("switch geom p3 r");
			a.addState("switch geom v3 x");
			a.addState("switch geom v3 y");
			a.addState("switch geom v3 z");
			a.addState("switch geom v4 x");
			a.addState("switch geom v4 y");
			a.addState("switch geom v4 z");
			a.addState("switch geom p4 x");
			a.addState("switch geom p4 y");
			a.addState("switch geom p4 z");
			a.addState("switch geom p4 r");
			a.addState("switch rad 2");
			a.addState("switch event");

			a.setFinal("ignore");
			a.setFinal("error");
			
			a.addRule("node", "ignore", "state1");
			a.addRule("description", "ignore", "description");
			a.addDefaultRule("ignore", "ignore");

			a.addRule("number", "state1", "state2");
			a.addRule("eol", "state1", "state1");
			a.addDefaultRule("state1", "error");

			a.addRule("number", "state2", "state3");
			a.addRule("eol", "state2", "state2");
			a.addDefaultRule("state2", "error");

			a.addRule("number", "state3", "error");
			a.addRule("eol", "state3", "state3");
			a.addDefaultRule("state3", "state4");

			a.addRule("inne", "state4", "ignore");
			a.addRule("track", "state4", "track");
			a.addRule("eol", "state4", "state4");
			a.addDefaultRule("state4", "error");

			// track
			a.addRule("normal", "track", "inside track 1");
			a.addRule("road", "track", "other track");
			a.addRule("switch", "track", "switch");
			a.addRule("river", "track", "other track");
			a.addRule("eol", "track", "track");
			a.addDefaultRule("track", "error");

			// Other track
			a.addRule("endtrack", "other track", "ignore");
			a.addDefaultRule("other track", "other track");

			// Track length
			a.addRule("number", "inside track 1", "inside track 2");
			a.addRule("eol", "inside track 1", "inside track 1");
			a.addDefaultRule("inside track 1", "error");

			// Track width
			a.addRule("number", "inside track 2", "inside track 3");
			a.addRule("eol", "inside track 2", "inside track 2");
			a.addDefaultRule("inside track 2", "error");

			// Track friction
			a.addRule("number", "inside track 3", "inside track 3a");
			a.addRule("eol", "inside track 3", "inside track 3");
			a.addDefaultRule("inside track 3", "error");

			// Track tapping
			a.addRule("number", "inside track 3a", "inside track 4");
			a.addRule("eol", "inside track 3a", "inside track 3a");
			a.addDefaultRule("inside track 3a", "error");

			// Track quality
			a.addRule("number", "inside track 4", "inside track 5");
			a.addRule("eol", "inside track 4", "inside track 4");
			a.addDefaultRule("inside track 4", "error");

			// Track damage
			a.addRule("number", "inside track 5", "inside track 6");
			a.addRule("eol", "inside track 5", "inside track 5");
			a.addDefaultRule("inside track 6", "error");

			// Track enviroment
			a.addRule("inne", "inside track 6", "inside track 7");
			a.addRule("eol", "inside track 6", "inside track 6");
			a.addDefaultRule("inside track 6", "error");

			// Track visbility
			a.addRule("vis", "inside track 7", "inside track 9");
			a.addRule("unvis", "inside track 7", "inside track 8");
			a.addRule("eol", "inside track 7", "inside track 7");
			a.addDefaultRule("inside track 7", "error");

			// Inside track 8
			a.addRule("number", "inside track 8", "track geom p1 1");
			a.addRule("inne", "inside track 8", "track tex 1");
			a.addRule("eol", "inside track 8", "inside track 8");
			a.addDefaultRule("inside track 8", "error");

			// Inside track 9
			a.addRule("inne", "inside track 9", "track tex 1");
			a.addRule("eol", "inside track 9", "inside track 9");
			a.addDefaultRule("inside track 9", "error");

			// Track texture 1
			a.addRule("number", "track tex 1", "track tex 2");
			a.addRule("eol", "track tex 1", "track tex 1");
			a.addDefaultRule("track tex 1", "error");

			// Track texture 2
			a.addRule("inne", "track tex 2", "track tex 3");
			a.addRule("eol", "track tex 2", "track tex 2");
			a.addDefaultRule("track tex 2", "error");

			// Track texture 3
			a.addRule("number", "track tex 3", "track tex 4");
			a.addRule("eol", "track tex 3", "track tex 3");
			a.addDefaultRule("track tex 3", "error");

			// Track texture 4
			a.addRule("number", "track tex 4", "track tex 5");
			a.addRule("eol", "track tex 4", "track tex 4");
			a.addDefaultRule("track tex 4", "error");
			
			// Track texture 5
			a.addRule("number", "track tex 5", "track tex 6");
			a.addRule("eol", "track tex 5", "track tex 5");
			a.addDefaultRule("track tex 5", "error");

			// Track texture 6
			a.addRule("number", "track tex 6", "track geom p1 1");
			a.addRule("eol", "track tex 6", "track tex 6");
			a.addDefaultRule("track tex 6", "error");

			// Track geometry Point 1 X
			a.addRule("number", "track geom p1 1", "track geom p1 2");
			a.addRule("eol", "track geom p1 1", "track geom p1 1");
			a.addDefaultRule("track geom p1 1", "error");

			// Track geometry Point 1 Y
			a.addRule("number", "track geom p1 2", "track geom p1 3");
			a.addRule("eol", "track geom p1 2", "track geom p1 2");
			a.addDefaultRule("track geom p1 2", "error");

			// Track geometry Point 1 Z
			a.addRule("number", "track geom p1 3", "track geom r1");
			a.addRule("eol", "track geom p1 3", "track geom p1 3");
			a.addDefaultRule("track geom p1 3", "error");

			// Track geometry Roll 1
			a.addRule("number", "track geom r1", "track geom v1 1");
			a.addRule("eol", "track geom r1", "track geom r1");
			a.addDefaultRule("track geom r1", "error");

			// Track geometry Vector 1 X
			a.addRule("number", "track geom v1 1", "track geom v1 2");
			a.addRule("eol", "track geom v1 1", "track geom v1 1");
			a.addDefaultRule("track geom v1 1", "error");

			// Track geometry Vector 1 Y
			a.addRule("number", "track geom v1 2", "track geom v1 3");
			a.addRule("eol", "track geom v1 2", "track geom v1 2");
			a.addDefaultRule("track geom v1 2", "error");

			// Track geometry Vector 1 Z
			a.addRule("number", "track geom v1 3", "track geom v2 1");
			a.addRule("eol", "track geom v1 3", "track geom v1 3");
			a.addDefaultRule("track geom v1 3", "error");

			// Track geometry Vector 2 X
			a.addRule("number", "track geom v2 1", "track geom v2 2");
			a.addRule("eol", "track geom v2 1", "track geom v2 1");
			a.addDefaultRule("track geom v2 1", "error");

			// Track geometry Vector 2 Y
			a.addRule("number", "track geom v2 2", "track geom v2 3");
			a.addRule("eol", "track geom v2 2", "track geom v2 2");
			a.addDefaultRule("track geom v2 2", "error");

			// Track geometry Vector 2 Z
			a.addRule("number", "track geom v2 3", "track geom p2 1");
			a.addRule("eol", "track geom v2 3", "track geom v2 3");
			a.addDefaultRule("track geom v2 3", "error");

			// Track geometry Point 2 X
			a.addRule("number", "track geom p2 1", "track geom p2 2");
			a.addRule("eol", "track geom p2 1", "track geom p2 1");
			a.addDefaultRule("track geom p2 1", "error");

			// Track geometry Point 2 Y
			a.addRule("number", "track geom p2 2", "track geom p2 3");
			a.addRule("eol", "track geom p2 2", "track geom p2 2");
			a.addDefaultRule("track geom p2 2", "error");

			// Track geometry Point 2 Z
			a.addRule("number", "track geom p2 3", "track geom r2");
			a.addRule("eol", "track geom p2 3", "track geom p2 3");
			a.addDefaultRule("track geom p2 3", "error");

			// Track geometry Roll 2
			a.addRule("number", "track geom r2", "track geom radius");
			a.addRule("eol", "track geom r2", "track geom r2");
			a.addDefaultRule("track geom r2", "error");

			// Track geometry Radius
			a.addRule("endtrack", "track geom radius", "ignore");
			a.addRule("eol", "track geom radius", "track geom radius");
			a.addDefaultRule("track geom radius", "track event");

			// End of track
			a.addRule("endtrack", "track event", "ignore");
			a.addRule("eol", "track event", "track event");
			a.addRule("inne", "track event", "track event");
			a.addRule("number", "track event", "track event");
			a.addDefaultRule("track event", "error");

			// description
			a.addRule("description", "description", "error");
			a.addRule("enddescription", "description", "ignore");
			a.addDefaultRule("description", "description");

			// Switch length
			a.addRule("number", "switch", "switch length");
			a.addRule("eol", "switch", "switch");
			a.addDefaultRule("switch", "error");

			// Switch width
			a.addRule("number", "switch length", "switch width");
			a.addRule("eol", "switch length", "switch length");
			a.addDefaultRule("switch length", "error");

			// Switch friction
			a.addRule("number", "switch width", "switch friction");
			a.addRule("eol", "switch width", "switch width");
			a.addDefaultRule("switch width", "error");

			// Switch tapping
			a.addRule("number", "switch friction", "switch tap");
			a.addRule("eol", "switch friction", "switch friction");
			a.addDefaultRule("switch friction", "error");

			// Switch quality
			a.addRule("number", "switch tap", "switch quality");
			a.addRule("eol", "switch tap", "switch tap");
			a.addDefaultRule("switch tap", "error");

			// Switch damage
			a.addRule("number", "switch quality", "switch damage");
			a.addRule("eol", "switch quality", "switch quality");
			a.addDefaultRule("switch quality", "error");

			// Switch enviroment
			a.addRule("inne", "switch damage", "switch env");
			a.addRule("eol", "switch damage", "switch damage");
			a.addDefaultRule("switch damage", "error");

			// Switch visibility
			a.addRule("vis", "switch env", "switch vis");
			a.addRule("unvis", "switch env", "switch unvis");
			a.addRule("eol", "switch env", "switch env");
			a.addDefaultRule("switch env", "error");

			// Switch vis
			a.addRule("inne", "switch vis", "switch tex 1");
			a.addRule("eol", "switch vis", "switch vis");
			a.addDefaultRule("switch vis", "error");

			// Switch unvis
			a.addRule("number", "switch unvis", "switch geom p1 x");
			a.addRule("inne", "switch unvis", "switch tex 1");
			a.addRule("eol", "switch unvis", "switch unvis");
			a.addDefaultRule("switch unvis", "error");

			// Switch texture length
			a.addRule("number", "switch tex 1", "switch tex 2");
			a.addRule("eol", "switch tex 1", "switch tex 1");
			a.addDefaultRule("switch tex 1", "error");

			// Switch second texture
			a.addRule("inne", "switch tex 2", "switch tex 3");
			a.addRule("eol", "switch tex 2", "switch tex 2");
			a.addDefaultRule("switch tex 2", "error");

			// Switch texture width
			a.addRule("number", "switch tex 3", "switch tex 4");
			a.addRule("eol", "switch tex 3", "switch tex 3");
			a.addDefaultRule("switch tex 3", "error");

			// Switch texture height
			a.addRule("number", "switch tex 4", "switch tex 5");
			a.addRule("eol", "switch tex 4", "switch tex 4");
			a.addDefaultRule("switch tex 4", "error");

			// Switch texture slope
			a.addRule("number", "switch tex 5", "switch tex 6");
			a.addRule("eol", "switch tex 5", "switch tex 5");
			a.addDefaultRule("switch tex 5", "error");

			// (transition) Switch geom p1 x
			a.addRule("number", "switch tex 6", "switch geom p1 x");
			a.addRule("eol", "switch tex 6", "switch tex 6");
			a.addDefaultRule("switch tex 6", "error");

			// Switch geom p1 y
			a.addRule("number", "switch geom p1 x", "switch geom p1 y");
			a.addRule("eol", "switch geom p1 x", "switch geom p1 x");
			a.addDefaultRule("switch geom p1 x", "error");

			// Switch geom p1 z
			a.addRule("number", "switch geom p1 y", "switch geom p1 z");
			a.addRule("eol", "switch geom p1 y", "switch geom p1 y");
			a.addDefaultRule("switch geom p1 y", "error");

			// Switch geom p1 r
			a.addRule("number", "switch geom p1 z", "switch geom p1 r");
			a.addRule("eol", "switch geom p1 z", "switch geom p1 z");
			a.addDefaultRule("switch geom p1 z", "error");

			// Switch geom v1 x
			a.addRule("number", "switch geom p1 r", "switch geom v1 x");
			a.addRule("eol", "switch geom p1 r", "switch geom p1 r");
			a.addDefaultRule("switch geom p1 r", "error");

			// Switch geom v1 y
			a.addRule("number", "switch geom v1 x", "switch geom v1 y");
			a.addRule("eol", "switch geom v1 x", "switch geom v1 x");
			a.addDefaultRule("switch geom v1 x", "error");

			// Switch geom v1 z
			a.addRule("number", "switch geom v1 y", "switch geom v1 z");
			a.addRule("eol", "switch geom v1 y", "switch geom v1 y");
			a.addDefaultRule("switch geom v1 y", "error");

			// Switch geom v2 x
			a.addRule("number", "switch geom v1 z", "switch geom v2 x");
			a.addRule("eol", "switch geom v1 z", "switch geom v1 z");
			a.addDefaultRule("switch geom v1 z", "error");

			// Switch geom v2 y
			a.addRule("number", "switch geom v2 x", "switch geom v2 y");
			a.addRule("eol", "switch geom v2 x", "switch geom v2 x");
			a.addDefaultRule("switch geom v2 x", "error");

			// Switch geom v2 z
			a.addRule("number", "switch geom v2 y", "switch geom v2 z");
			a.addRule("eol", "switch geom v2 y", "switch geom v2 y");
			a.addDefaultRule("switch geom v2 y", "error");

			// Switch geom p2 x
			a.addRule("number", "switch geom v2 z", "switch geom p2 x");
			a.addRule("eol", "switch geom v2 z", "switch geom v2 z");
			a.addDefaultRule("switch geom v2 z", "error");

			// Switch geom p2 y
			a.addRule("number", "switch geom p2 x", "switch geom p2 y");
			a.addRule("eol", "switch geom p2 x", "switch geom p2 x");
			a.addDefaultRule("switch geom p2 x", "error");

			// Switch geom p2 z
			a.addRule("number", "switch geom p2 y", "switch geom p2 z");
			a.addRule("eol", "switch geom p2 y", "switch geom p2 y");
			a.addDefaultRule("switch geom p2 y", "error");

			// Switch geom p2 r
			a.addRule("number", "switch geom p2 z", "switch geom p2 r");
			a.addRule("eol", "switch geom p2 z", "switch geom p2 z");
			a.addDefaultRule("switch geom p2 z", "error");

			// Switch geom radius 1
			a.addRule("number", "switch geom p2 r", "switch rad 1");
			a.addRule("eol", "switch geom p2 r", "switch geom p2 r");
			a.addDefaultRule("switch geom p2 r", "error");

			// Switch geom p3 x
			a.addRule("number", "switch rad 1", "switch geom p3 x");
			a.addRule("eol", "switch rad 1", "switch rad 1");
			a.addDefaultRule("switch rad 1", "error");

			// Switch geom p3 y
			a.addRule("number", "switch geom p3 x", "switch geom p3 y");
			a.addRule("eol", "switch geom p3 x", "switch geom p3 x");
			a.addDefaultRule("switch geom p3 x", "error");

			// Switch geom p3 z
			a.addRule("number", "switch geom p3 y", "switch geom p3 z");
			a.addRule("eol", "switch geom p3 y", "switch geom p3 y");
			a.addDefaultRule("switch geom p3 y", "error");

			// Switch geom p3 r
			a.addRule("number", "switch geom p3 z", "switch geom p3 r");
			a.addRule("eol", "switch geom p3 z", "switch geom p3 z");
			a.addDefaultRule("switch geom p3 z", "error");

			// Switch geom v3 x
			a.addRule("number", "switch geom p3 r", "switch geom v3 x");
			a.addRule("eol", "switch geom p3 r", "switch geom p3 r");
			a.addDefaultRule("switch geom p3 r", "error");

			// Switch geom v3 y
			a.addRule("number", "switch geom v3 x", "switch geom v3 y");
			a.addRule("eol", "switch geom v3 x", "switch geom v3 x");
			a.addDefaultRule("switch geom v3 x", "error");

			// Switch geom v3 z
			a.addRule("number", "switch geom v3 y", "switch geom v3 z");
			a.addRule("eol", "switch geom v3 y", "switch geom v3 y");
			a.addDefaultRule("switch geom v3 y", "error");

			// Switch geom v4 x
			a.addRule("number", "switch geom v3 z", "switch geom v4 x");
			a.addRule("eol", "switch geom v3 z", "switch geom v3 z");
			a.addDefaultRule("switch geom v3 z", "error");

			// Switch geom v4 y
			a.addRule("number", "switch geom v4 x", "switch geom v4 y");
			a.addRule("eol", "switch geom v4 x", "switch geom v4 x");
			a.addDefaultRule("switch geom v4 x", "error");

			// Switch geom v4 z
			a.addRule("number", "switch geom v4 y", "switch geom v4 z");
			a.addRule("eol", "switch geom v4 y", "switch geom v4 y");
			a.addDefaultRule("switch geom v4 y", "error");

			// Switch geom p4 x
			a.addRule("number", "switch geom v4 z", "switch geom p4 x");
			a.addRule("eol", "switch geom v4 z", "switch geom v4 z");
			a.addDefaultRule("switch geom v4 z", "error");

			// Switch geom p4 y
			a.addRule("number", "switch geom p4 x", "switch geom p4 y");
			a.addRule("eol", "switch geom p4 x", "switch geom p4 x");
			a.addDefaultRule("switch geom p4 x", "error");

			// Switch geom p4 z
			a.addRule("number", "switch geom p4 y", "switch geom p4 z");
			a.addRule("eol", "switch geom p4 y", "switch geom p4 y");
			a.addDefaultRule("switch geom p4 y", "error");

			// Switch geom p4 r
			a.addRule("number", "switch geom p4 z", "switch geom p4 r");
			a.addRule("eol", "switch geom p4 z", "switch geom p4 z");
			a.addDefaultRule("switch geom p4 z", "error");

			// Switch radius 2
			a.addRule("number", "switch geom p4 r", "switch rad 2");
			a.addRule("eol", "switch geom p4 r", "switch geom p4 r");
			a.addDefaultRule("switch geom p4 r", "error");

			// Switch events
			a.addRule("endtrack", "switch rad 2", "ignore");
			a.addRule("eol", "switch rad 2", "switch rad 2");
			a.addDefaultRule("switch rad 2", "switch event");

			// End of track
			a.addRule("endtrack", "switch event", "ignore");
			a.addRule("eol", "switch event", "switch event");
			a.addRule("inne", "switch event", "switch event");
			a.addRule("number", "switch event", "switch event");
			a.addDefaultRule("switch event", "error");

			a.setStart("ignore");
		}
		catch (AutomatonException ae)
		{
			ae.printStackTrace();
		}
	}

	/**
	 * Gets an automaton word.
	 *
	 * @param str The string to recognize.
	 * @return Automaton word.
	 */
	private String getWord(String str)
	{
		str.toLowerCase();
		if (str.equals("node"))
			return "node";
		else if (str.equals("track"))
			return "track";
		else if (str.equals("endtrack"))
			return "endtrack";
		else if (str.equals("description"))
			return "description";
		else if (str.equals("enddescription"))
			return "enddescription";
		else if (str.equals("normal"))
			return "normal";
		else if (str.equals("road"))
			return "road";
		else if (str.equals("switch"))
			return "switch";
		else if (str.equals("river"))
			return "river";
		else if (str.equals("vis"))
			return "vis";
		else if (str.equals("novis") || str.equals("unvis"))
			return "unvis";
		else
			return "inne";
	}

	/**
	 * Removes new lines at beginning of scenery desription.
	 *
	 * @param sb String buffer that contains description.
	 */
	private void removeNewLines(StringBuffer sb)
	{
		while (sb.length() > 0 && sb.charAt(0) == '\n')
			sb.deleteCharAt(0);
	}

	/**
	 * Writes scenery to the output stream.
	 *
	 * @param out The output stream.
	 * @param sc Scenery to write.
	 * @exception IOException Thrown when some I/O errors ocuured.
	 */
	public void write(OutputStream out, Scenery sc) throws IOException
	{
		OutputStreamWriter osw = new OutputStreamWriter(out);
		write(osw, sc);
		osw.flush();
	}

	/**
	 * Writes scenery to the writer.
	 *
	 * @param out The output stream.
	 * @param sc Scenery to write.
	 * @exception IOException Thrown when some I/O errors occured.
	 */
	public void write(Writer out, Scenery sc) throws IOException
	{
		String desc;
		PrintWriter pOut = new PrintWriter(out);

		// Write description
		if ((desc = sc.getDescription()) != null)
		{
			pOut.println("description");
			pOut.println(desc);
			pOut.println("enddescription");
			pOut.println();
		}

		// Write tracks
		sc.readLock();
		try
		{
			Iterator i;

			i = sc.getIterator(Track.class);
			while (i.hasNext())
			{
				printTrack(pOut, (Track) i.next());
			}

			i = sc.getIterator(Switch.class);
			while (i.hasNext())
			{
				printSwitch(pOut, (Switch) i.next());
			}
		}
		finally
		{
			sc.readUnlock();
		}

		pOut.println("firstinit");
	}

	/**
	 * Prints track to the print stream.
	 */
	private void printTrack(PrintWriter pOut, Track track)
	{
		pOut.print("node -1 0 ");
		pOut.print( track.getName() == null ? "none" : track.getName() );
		pOut.print(" track normal ");
		pOut.print( track.getLength() );
		pOut.print(' ');
		pOut.print( track.getWidth() );
		pOut.print(' ');
		pOut.print( track.getFriction() );
		pOut.print(' ');
		pOut.print( track.getTap() );
		pOut.print(' ');
		pOut.print( track.getQuality() );
		pOut.print(' ');
		pOut.print( track.getDamage() );
		pOut.print(' ');
		pOut.print( track.getEnviroment() == null ? "flat" : track.getEnviroment() );
		pOut.print(' ');
		pOut.println( track.isVisible() ? "vis" : "novis" );

		if ( track.getTexture() != null && track.isVisible() )
		{
			pOut.print( track.getTexture() );
			pOut.print(' ');
			pOut.print( track.getTextureLength() );
			pOut.print(' ');
			pOut.print( track.getSecondTexture() );
			pOut.print(' ');
			pOut.print( track.getTextureHeight() );
			pOut.print(' ');
			pOut.print( track.getTextureWidth() );
			pOut.print(' ');
			pOut.println( track.getTextureSlope() );
		}

		pOut.print("  ");
		pOut.print( track.getPoint1().x );
		pOut.print(' ');
		pOut.print( track.getPoint1().y );
		pOut.print(' ');
		pOut.print( track.getPoint1().z );
		pOut.print(' ');
		pOut.print( track.getRoll1() );
		pOut.println(" // Point 1");

		pOut.print("  ");
		pOut.print( track.getVector1().x );
		pOut.print(' ');
		pOut.print( track.getVector1().y );
		pOut.print(' ');
		pOut.print( track.getVector1().z );
		pOut.println(" // Control vector 1");

		pOut.print("  ");
		pOut.print( track.getVector2().x );
		pOut.print(' ');
		pOut.print( track.getVector2().y );
		pOut.print(' ');
		pOut.print( track.getVector2().z );
		pOut.println(" // Control vector 2");

		pOut.print("  ");
		pOut.print( track.getPoint2().x );
		pOut.print(' ');
		pOut.print( track.getPoint2().y );
		pOut.print(' ');
		pOut.print( track.getPoint2().z );
		pOut.print(' ');
		pOut.print( track.getRoll2() );
		pOut.println(" // Point 2");
		pOut.print("  ");
		pOut.println( track.getRadius() );

		pOut.println("endtrack");
		pOut.println();
	}

	/**
	 * Prints switch.
	 */
	public void printSwitch(PrintWriter pOut, Switch sw)
	{
		pOut.print("node -1 0 ");
		pOut.print( sw.getName() == null ? "none" : sw.getName() );
		pOut.print(" track switch ");
		pOut.print( sw.getLength() );
		pOut.print(' ');
		pOut.print( sw.getWidth() );
		pOut.print(' ');
		pOut.print( sw.getFriction() );
		pOut.print(' ');
		pOut.print( sw.getTap() );
		pOut.print(' ');
		pOut.print( sw.getQuality() );
		pOut.print(' ');
		pOut.print( sw.getDamage() );
		pOut.print(' ');
		pOut.print( sw.getEnviroment() == null ? "flat" : sw.getEnviroment() );
		pOut.print(' ');
		pOut.println( sw.isVisible() ? "vis" : "novis" );

		if ( sw.getTexture() != null && sw.isVisible() )
		{
			pOut.print( sw.getTexture() );
			pOut.print(' ');
			pOut.print( sw.getTextureLength() );
			pOut.print(' ');
			pOut.print( sw.getSecondTexture() );
			pOut.print(' ');
			pOut.print( sw.getTextureHeight() );
			pOut.print(' ');
			pOut.print( sw.getTextureWidth() );
			pOut.print(' ');
			pOut.println( sw.getTextureSlope() );
		}

		pOut.print("  ");
		pOut.print( sw.getPoint1().x );
		pOut.print(' ');
		pOut.print( sw.getPoint1().y );
		pOut.print(' ');
		pOut.print( sw.getPoint1().z );
		pOut.print(' ');
		pOut.print( sw.getRoll1() );
		pOut.println(" // Point 1");

		pOut.print("  ");
		pOut.print( sw.getVector1().x );
		pOut.print(' ');
		pOut.print( sw.getVector1().y );
		pOut.print(' ');
		pOut.print( sw.getVector1().z );
		pOut.println(" // Control vector 1");

		pOut.print("  ");
		pOut.print( sw.getVector2().x );
		pOut.print(' ');
		pOut.print( sw.getVector2().y );
		pOut.print(' ');
		pOut.print( sw.getVector2().z );
		pOut.println(" // Control vector 2");

		pOut.print("  ");
		pOut.print( sw.getPoint2().x );
		pOut.print(' ');
		pOut.print( sw.getPoint2().y );
		pOut.print(' ');
		pOut.print( sw.getPoint2().z );
		pOut.print(' ');
		pOut.print( sw.getRoll2() );
		pOut.println(" // Point 2");

		pOut.print("  ");
		pOut.println( sw.getRadius() );

		// Tor zwrotny
		pOut.print("  ");
		pOut.print( sw.getPoint1().x );
		pOut.print(' ');
		pOut.print( sw.getPoint1().y );
		pOut.print(' ');
		pOut.print( sw.getPoint1().z );
		pOut.print(' ');
		pOut.print( sw.getSwitchedRoll() );
		pOut.println(" // Point 3");

		pOut.print("  ");
		pOut.print( sw.getSwitchedVector().x );
		pOut.print(' ');
		pOut.print( sw.getSwitchedVector().y );
		pOut.print(' ');
		pOut.print( sw.getSwitchedVector().z );
		pOut.println(" // Control vector 3");

		pOut.print("  ");
		pOut.print( sw.getVector3().x );
		pOut.print(' ');
		pOut.print( sw.getVector3().y );
		pOut.print(' ');
		pOut.print( sw.getVector3().z );
		pOut.println(" // Control vector 4");

		pOut.print("  ");
		pOut.print( sw.getPoint3().x );
		pOut.print(' ');
		pOut.print( sw.getPoint3().y );
		pOut.print(' ');
		pOut.print( sw.getPoint3().z );
		pOut.print(' ');
		pOut.print( sw.getRoll3() );
		pOut.println(" // Point 4");

		pOut.print("  ");
		pOut.println( sw.getSwitchedRadius() );

		pOut.println("endtrack");
		pOut.println();
	}

	/*
	 * --------------- Scenery creating ------------------
	 */
	
	/**
	 * Creates default scenery.
	 *
	 * @return Scenery.
	 */
	public Scenery createDefaultScenery()
	{
		return new Scenery();
	}

	/*
	 * --------------- Inner classes -----------------
	 */
	
	/**
	 * The exception when scenery file is corrupted.
	 */
	public static class SceneryException extends IOException
	{
		/**
		 * Throws new scenery corruption exception with message
		 * and line number where an error occured.
		 *
		 * @param message Message.
		 * @param line Line number in corrupted scenery file.
		 */
		public SceneryException(String message, int line)
		{
			super(message);
			this.line = line;
		}

		/**
		 * Throws new scenery corruption exception with
		 * line number in scenery file where an error occured.
		 *
		 * @param line Line number.
		 */
		public SceneryException(int line)
		{
			super();
			this.line = line;
		}

		/**
		 * Gets line number of scenery file where the exception
		 * occured.
		 *
		 * @return Line number.
		 */
		public int getLine()
		{
			return line;
		}

		/*
		 * ------- Members ---------
		 */
		
		/** Line number of an exception. */
		private int line;
	}
}
