/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.sce;

import org.jet.tracks.Track;
import javax.vecmath.*;

/**
 * Track for read only use.
 * All methods are immutable.
 *
 * @author Adam Domaski
 * @version 0.1 3/02/2004
 */
final class ImmutableTrack extends Track
{
	/**
	 * Constructs new immutuable track copying all data from
	 * original track.
	 *
	 * @param track Original track.
	 */
	public ImmutableTrack(Track track)
	{
		name = track.getName();
		node1 = (Point3d) track.getPoint1().clone();
		node2 = (Point3d) track.getPoint2().clone();
		vec1 = (Vector3d) track.getVector1().clone();
		vec2 = (Vector3d) track.getVector2().clone();
		roll1 = track.getRoll1();
		roll2 = track.getRoll2();
		radius = track.getRadius();
		length = track.getLength();
		width = track.getWidth();
		friction = track.getFriction();
		tapDistance = track.getTap();
		quality = track.getQuality();
		damage = track.getDamage();
		enviroment = track.getEnviroment();
		visibility = track.isVisible();
		texture = track.getTexture();
		textureLength = track.getTextureLength();
		secondTexture = track.getSecondTexture();
		texHeight = track.getTextureHeight();
		texWidth = track.getTextureWidth();
		texSlope = track.getTextureSlope();
	}

	/*
	 * ------- Overriding setXXX methods ---------
	 */

	/**
	 * Does nothing.
	 */
	public void setName(String name)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setPoint1(Point3d p)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setPoint1(double[] array)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setPoint1(double x, double y, double z)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setPoint2(Point3d p)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setPoint2(double x, double y, double z)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setPoint2(double[] array)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setVector1(Vector3d v)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setVector1(double x, double y, double z)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setVector1(double[] array)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setVector2(Vector3d v)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setVector2(double x, double y, double z)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setVector2(double[] array)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setRoll1(double roll)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setRoll2(double roll)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setLength(double d)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setWidth(double d)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setFriction(double d)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setTap(double d)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setQuality(int i)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setDamage(int i)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setEnviroment(String enviroment)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setVisible(boolean b)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setRadius(double d)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setTexture(String name)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setTextureLength(double d)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setSecondTexture(String name)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setTextureHeight(double d)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setTextureWidth(double d)
	{
	}

	/**
	 * Does nothing.
	 */
	public void setTextureSlope(double d)
	{
	}
}
