/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.tracks;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.vecmath.*;

import org.jet.*;
import org.jet.sce.*;

/**
 * Custom switch properties dialog.
 *
 * @author Adam Domaski
 * @version 0.1 11/03/2004
 */
public class CustomSwitchProperties extends SwitchProperties
{
	public CustomSwitchProperties(Jet app)
	{
		super(app.getFrame());

		application = app;
		editable = true;

		resource = JetResources.getInstance();
		setTitle( resource.translate("CustomSwitchProperties.title") );

		setResizable(false);
		setDefaultCloseOperation(HIDE_ON_CLOSE);

		initialize();
		fillIn();

		pack();
		centerDialog();
		setVisible(true);
	}

	/**
	 * Initialize dialog controls.
	 */
	private void initialize()
	{
		JPanel pane = new JPanel();
		pane.setLayout(new BorderLayout());

		JPanel container = createContent();
		pane.add(container);

		JPanel controlPanel = new JPanel();		
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		controlPanel.setLayout(gb);
		controlPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		JButton okButton = new JButton( resource.translate("SwitchProperties.ok") );
		JButton caButton = new JButton( resource.translate("SwitchProperties.cancel") );

		caButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae)
			{
				dispose();
			}
		});

		getRootPane().setDefaultButton(okButton);
		okButton.addActionListener(new OKAction());

		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.EAST;
		c.insets = new Insets(0, 0, 0, 5);
		gb.setConstraints(okButton, c);
		controlPanel.add(okButton);

		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 0.0;
		c.insets = new Insets(0, 0, 0, 0);
		gb.setConstraints(caButton, c);
		controlPanel.add(caButton);

		normalize(okButton, caButton);
		okButton.requestFocusInWindow();

		pane.add(controlPanel, BorderLayout.SOUTH);

		getContentPane().add(pane);
	}

	/**
	 * Creates main content.
	 */
	private JPanel createContent()
	{
		JPanel jPanel;
		JLabel jLabel;
		JTextField jText;

		JPanel pane = new JPanel();
		pane.setName("SwitchPropertiesPanel");
		pane.setBorder( BorderFactory.createEmptyBorder(1, 1, 1, 1) );
		
		JTabbedPane jTabbedPane = new JTabbedPane();
		jTabbedPane.setName("SwitchPropertiesTabbedPane");

		JPanel jGeneralPanel = new JPanel();
		jGeneralPanel.setName("GeneralPanel");
		jGeneralPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		GridBagLayout rootGb = new GridBagLayout();
		GridBagConstraints rootC = new GridBagConstraints();
		rootC.anchor = GridBagConstraints.WEST;
		rootC.insets = new Insets(0, 0, 5, 0);
		rootC.fill = GridBagConstraints.BOTH;
		jGeneralPanel.setLayout(rootGb);

		// General panel (name, friction, etc.)

		jPanel = new JPanel();
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		Insets insets = new Insets(1, 2, 1, 2);
		jPanel.setLayout(gb);
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		
		jLabel = new JLabel();
		jLabel.setName("NameLabel");
		jLabel.setText( resource.translate("SwitchProperties.name") );
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.WEST;
		c.insets = insets;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createName();
		jText.setName("NameTextBox");
		jText.setColumns(10);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("WidthLabel");
		jLabel.setText( resource.translate("SwitchProperties.width") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createWidth();
		jText.setName("WidthTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("FrictionLabel");
		jLabel.setText( resource.translate("SwitchProperties.friction") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createFriction();
		jText.setName("FrictionTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TapLabel");
		jLabel.setText( resource.translate("SwitchProperties.tap") );
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTap();
		jText.setName("TapTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("QualityLabel");
		jLabel.setText( resource.translate("SwitchProperties.quality") );
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createQuality();
		jText.setName("QualityTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("DamageLabel");
		jLabel.setText( resource.translate("SwitchProperties.damage") );
		c.gridx = 2;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createDamage();
		jText.setName("DamageTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jPanel.setFocusTraversalPolicy(new GeneralTabFocusTraversal());
		rootC.gridx = 0;
		rootC.gridy = 0;
		rootC.gridwidth = 2;
		rootGb.setConstraints(jPanel, rootC);
		jGeneralPanel.add(jPanel);

		// Wasnoci torw skadowych rozjazdu

		// Tor prosty

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder(
				BorderFactory.createEtchedBorder(), 
				resource.translate("CustomSwitchProperties.straightTrack")),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		// Dugo
		jLabel = new JLabel();
		jLabel.setName("LengthRLabel");
		jLabel.setText( resource.translate("CustomSwitchProperties.lengthR") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createLength();
		jText.setName("LengthRTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("RadiusRLabel");
		jLabel.setText( resource.translate("CustomSwitchProperties.radiusR") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRadius();
		jText.setName("RadiusRTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("AngleRLabel");
		jLabel.setText( resource.translate("CustomSwitchProperties.angleR") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createDegreeFormatTextBox();
		jText.setName("AngleRTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		rootC.gridx = 0;
		rootC.gridy = 1;
		rootC.gridwidth = 1;
		rootGb.setConstraints(jPanel, rootC);
		jGeneralPanel.add(jPanel);

		// Tor zwrotny

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder(
				BorderFactory.createEtchedBorder(), 
				resource.translate("CustomSwitchProperties.switchedTrack")),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		// Dugo
		jLabel = new JLabel();
		jLabel.setName("LengthSLabel");
		jLabel.setText( resource.translate("CustomSwitchProperties.lengthS") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createLength();
		jText.setName("LengthSTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("RadiusSLabel");
		jLabel.setText( resource.translate("CustomSwitchProperties.radiusS") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRadius();
		jText.setName("RadiusSTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("AngleSLabel");
		jLabel.setText( resource.translate("CustomSwitchProperties.angleS") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createDegreeFormatTextBox();
		jText.setName("AngleSTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		rootC.gridx = 1;
		rootC.gridy = 1;
		rootGb.setConstraints(jPanel, rootC);
		jGeneralPanel.add(jPanel);

		// General panel

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		// Tor prawy / lewy
		jLabel = new JLabel();
		jLabel.setName("TwirlLabel");
		jLabel.setText( resource.translate("CustomSwitchProperties.twirl") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		JPanel jRightArc = new JPanel();
		jRightArc.setBorder( BorderFactory.createEtchedBorder() );
		jRightArc.setLayout( new GridLayout(1, 2) );

		JRadioButton jRadioButton;
		ButtonGroup bGroup = new ButtonGroup();

		jRadioButton = new JRadioButton();
		jRadioButton.setName("LeftRadioButton");
		jRadioButton.setText( resource.translate("CustomSwitchProperties.left") );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jRightArc.add(jRadioButton);

		jRadioButton = new JRadioButton();
		jRadioButton.setName("RightRadioButton");
		jRadioButton.setText( resource.translate("CustomSwitchProperties.right") );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jRightArc.add(jRadioButton);

		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jRightArc, c);
		jPanel.add(jRightArc);

		jLabel = new JLabel();
		jLabel.setName("EnviromentLabel");
		jLabel.setText( resource.translate("SwitchProperties.enviroment") );
		c.gridx = 0;
		c.gridy = 1;
		c.insets = insets;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		JComboBox jComboBox = new JComboBox();
		jComboBox.setName("EnviromentComboBox");
		jComboBox.addItem(Track.BANK);
		jComboBox.addItem(Track.BRIDGE);
		jComboBox.addItem(Track.CANYON);
		jComboBox.addItem(Track.FLAT);
		jComboBox.addItem(Track.MOUNTAINS);
		jComboBox.addItem(Track.TUNNEL);
		controls.putComponent(jComboBox);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		rootC.gridx = 0;
		rootC.gridy = 2;
		rootC.gridwidth = 2;
		rootGb.setConstraints(jPanel, rootC);
		jGeneralPanel.add(jPanel);

		jTabbedPane.addTab(resource.translate("SwitchProperties.general"), jGeneralPanel);

		// Tekstury

		JPanel jTexturesPanel = new JPanel();
		jTexturesPanel.setName("TexturesPanel");
		jTexturesPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		jTexturesPanel.setLayout(new GridLayout(2, 1, 0, 5));

		/* Tekstury - grny panel */

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("VisibilityLabel");
		jLabel.setText( resource.translate("SwitchProperties.visibility") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		JPanel jVisibilityPanel = new JPanel();
		jVisibilityPanel.setBorder( BorderFactory.createEtchedBorder() );
		jVisibilityPanel.setLayout( new GridLayout(1, 2) );

		bGroup = new ButtonGroup();
		jRadioButton = new JRadioButton();
		jRadioButton.setName("VisibleRadioButton");
		jRadioButton.setText( resource.translate("SwitchProperties.visible") );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jVisibilityPanel.add(jRadioButton);

		jRadioButton = new JRadioButton();
		jRadioButton.setName("InvisibleRadioButton");
		jRadioButton.setText( resource.translate("SwitchProperties.invisible") );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jVisibilityPanel.add(jRadioButton);

		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jVisibilityPanel, c);
		jPanel.add(jVisibilityPanel);
		
		jTexturesPanel.add(jPanel);

		/* Tekstury - dolny panel */

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("TextureLabel");
		jLabel.setText( resource.translate("SwitchProperties.texture") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jComboBox = createTexture();
		jComboBox.setName("TextureComboBox");
		controls.putComponent(jComboBox);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jLabel = new JLabel();
		jLabel.setName("TextureLengthLabel");
		jLabel.setText( resource.translate("SwitchProperties.textureLength") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureLength();
		jText.setName("TextureLengthEditBox");
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("SecondTextureLabel");
		jLabel.setText( resource.translate("SwitchProperties.secondTexture") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jComboBox = createSecondTexture();
		jComboBox.setName("SecondTextureComboBox");
		controls.putComponent( jComboBox );
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jLabel = new JLabel();
		jLabel.setName("TextureWidthLabel");
		jLabel.setText( resource.translate("SwitchProperties.textureWidth") );
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureWidth();
		jText.setName("TextureWidthTextBox");
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TextureHeightLabel");
		jLabel.setText( resource.translate("SwitchProperties.textureHeight") );
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureHeight();
		jText.setName("TextureHeightTextBox");
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TextureSlopeLabel");
		jLabel.setText( resource.translate("SwitchProperties.textureSlope") );
		c.gridx = 2;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureSlope();
		jText.setName("TextureSlopeTextBox");
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jTexturesPanel.add(jPanel);

		jTabbedPane.addTab(resource.translate("SwitchProperties.textures"), jTexturesPanel);

		/* Geometria */
		JPanel jTempPanel = new JPanel();
		jTempPanel.setLayout(new GridBagLayout());
		GridBagConstraints c3 = new GridBagConstraints();
		c3.gridx = 0; c3.gridy = 0; c3.fill = GridBagConstraints.BOTH;

		JPanel jGeometryPanel = new JPanel();
		jGeometryPanel.setName("GeometryPanel");
		jGeometryPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		GridBagLayout gb2 = new GridBagLayout();
		GridBagConstraints c2 = new GridBagConstraints();
		jGeometryPanel.setLayout(gb2);

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Point1Label");
		jLabel.setText( resource.translate("SwitchProperties.point1") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point1XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1YTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Vector1Label");
		jLabel.setText( resource.translate("SwitchProperties.vector1") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Vector1XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector1YTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector1ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Roll1Label");
		jLabel.setText( resource.translate("SwitchProperties.roll1") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("Roll1TextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		c2.gridx = 0;
		c2.gridy = 0;
		c2.insets = new Insets(0, 0, 5, 5);
		gb2.setConstraints(jPanel, c2);
		jGeometryPanel.add(jPanel);

		/* Point 2 */
		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Vector2Label");
		jLabel.setText( resource.translate("SwitchProperties.vector2") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Vector2XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setEnabled( false );
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector2ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Point2Label");
		jLabel.setText( resource.translate("SwitchProperties.point2") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point2XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point2YTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point2ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Roll2Label");
		jLabel.setText( resource.translate("SwitchProperties.roll2") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("Roll2TextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		c2.gridx = 0;
		c2.gridy = 1;
		c2.insets = new Insets(0, 0, 5, 5);
		gb2.setConstraints(jPanel, c2);
		jGeometryPanel.add(jPanel);

		/* Point 3 */
		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Point1RLabel");
		jLabel.setText( resource.translate("SwitchProperties.point1r") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point1RXTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1RYTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1RZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("VectorSLabel");
		jLabel.setText( resource.translate("SwitchProperties.vectorS") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("VectorSXTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("VectorSYTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("VectorSZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("RollSLabel");
		jLabel.setText( resource.translate("SwitchProperties.rollS") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("RollSTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		c2.gridx = 1;
		c2.gridy = 0;
		c2.insets = new Insets(0, 0, 5, 0);
		gb2.setConstraints(jPanel, c2);
		jGeometryPanel.add(jPanel);

		/* Point 4 */
		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Vector3Label");
		jLabel.setText( resource.translate("SwitchProperties.vector3") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Vector3XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector3YTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector3ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Point3Label");
		jLabel.setText( resource.translate("SwitchProperties.point3") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point3XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point3YTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point3ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Roll3Label");
		jLabel.setText( resource.translate("SwitchProperties.roll3") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("Roll3TextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		c2.gridx = 1;
		c2.gridy = 1;
		gb2.setConstraints(jPanel, c2);
		jGeometryPanel.add(jPanel);

		((GridBagLayout) jTempPanel.getLayout()).setConstraints(jGeometryPanel, c3);
		jTempPanel.add(jGeometryPanel);

		jTabbedPane.addTab(resource.translate("SwitchProperties.geometry"), jTempPanel);

		// Uchwyty

		JPanel jHandlers = new JPanel();
		
		JToggleButton jToggle;
		ButtonGroup handlersGroup = new ButtonGroup();

		jToggle = new JToggleButton();
		jToggle.setName("CommonHandlerButton");
		jToggle.setText( resource.translate("CustomSwitchProperties.commonHandler") );
		controls.putComponent(jToggle);
		handlersGroup.add(jToggle);
		jHandlers.add(jToggle);

		jToggle = new JToggleButton();
		jToggle.setName("StraightHandlerButton");
		jToggle.setText( resource.translate("CustomSwitchProperties.straightHandler") );
		controls.putComponent(jToggle);
		handlersGroup.add(jToggle);
		jHandlers.add(jToggle);

		jToggle = new JToggleButton();
		jToggle.setName("SideHandlerButton");
		jToggle.setText( resource.translate("CustomSwitchProperties.sideHandler") );
		controls.putComponent(jToggle);
		handlersGroup.add(jToggle);
		jHandlers.add(jToggle);

		jTabbedPane.addTab(resource.translate("CustomSwitchProperties.handlers"), jHandlers);

		pane.add(jTabbedPane);

		return pane;
	}

	private void fillIn()
	{
		JTextField			jText;
		JFormattedTextField jFormat;
		JRadioButton		jRadioButton;
		JComboBox			jComboBox;
		JToggleButton		jToggle;

		Configuration config = application.config;

		jFormat = (JFormattedTextField) controls.getComponent("LengthRTextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.length1") );

		jFormat = (JFormattedTextField) controls.getComponent("RadiusRTextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.radius1") );

		jFormat = (JFormattedTextField) controls.getComponent("LengthSTextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.length2") );
		double len2 = config.getDouble("CustomSwitchProperties.length2");

		jFormat = (JFormattedTextField) controls.getComponent("RadiusSTextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.radius2") );
		double rad2 = config.getDouble("CustomSwitchProperties.radius2");

		jFormat = (JFormattedTextField) controls.getComponent("AngleSTextBox");
		jFormat.setValue( new Double(Math.toDegrees(len2 / rad2)) );

		jFormat = (JFormattedTextField) controls.getComponent("WidthTextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.width") );

		jFormat = (JFormattedTextField) controls.getComponent("TapTextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.tap") );

		jFormat = (JFormattedTextField) controls.getComponent("FrictionTextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.friction") );

		jFormat = (JFormattedTextField) controls.getComponent("QualityTextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.quality") );

		jFormat = (JFormattedTextField) controls.getComponent("DamageTextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.damage") );

		boolean bool = ((Boolean) config.get("CustomSwitchProperties.right")).booleanValue();
		if (bool)
			jRadioButton = (JRadioButton) controls.getComponent("RightRadioButton");
		else
			jRadioButton = (JRadioButton) controls.getComponent("LeftRadioButton");
		jRadioButton.setSelected( true );

		jComboBox = (JComboBox) controls.getComponent("EnviromentComboBox");
		jComboBox.setSelectedItem( config.get("CustomSwitchProperties.enviroment") );

		bool = ((Boolean) config.get("CustomSwitchProperties.visibility")).booleanValue();
		if (bool)
			jRadioButton = (JRadioButton) controls.getComponent("VisibleRadioButton");
		else
			jRadioButton = (JRadioButton) controls.getComponent("InvisibleRadioButton");
		jRadioButton.setSelected( true );

		jComboBox = (JComboBox) controls.getComponent("TextureComboBox");
		jComboBox.setSelectedItem( config.get("CustomSwitchProperties.texture") );

		jFormat = (JFormattedTextField) controls.getComponent("TextureLengthEditBox");
		jFormat.setValue( config.get("CustomSwitchProperties.texLength") );

		jComboBox = (JComboBox) controls.getComponent("SecondTextureComboBox");
		jComboBox.setSelectedItem( config.get("CustomSwitchProperties.secondTexture") );

		jFormat = (JFormattedTextField) controls.getComponent("TextureWidthTextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.texWidth") );

		jFormat = (JFormattedTextField) controls.getComponent("TextureHeightTextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.texHeight") );

		jFormat = (JFormattedTextField) controls.getComponent("TextureSlopeTextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.texSlope") );

		jFormat = (JFormattedTextField) controls.getComponent("Roll1TextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.roll1") );

		jFormat = (JFormattedTextField) controls.getComponent("Roll2TextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.roll2") );

		jFormat = (JFormattedTextField) controls.getComponent("Roll3TextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.roll3") );

		jFormat = (JFormattedTextField) controls.getComponent("RollSTextBox");
		jFormat.setValue( config.get("CustomSwitchProperties.rollS") );

		int handlerConf = ((Integer) config.get("CustomSwitchProperties.handler")).
			intValue();
		if (handlerConf == TrackFactory.HANDLER_LEFT)
			jToggle = (JToggleButton) controls.getComponent("CommonHandlerButton");
		else if (handlerConf == TrackFactory.HANDLER_RIGHT)
			jToggle = (JToggleButton) controls.getComponent("StraightHandlerButton");
		else
			jToggle = (JToggleButton) controls.getComponent("SideHandlerButton");			
		jToggle.setSelected(true);

		setConstraints();
		setListeners();
	}

	/**
	 * Sets the constraints for text fields.
	 */
	private void setConstraints()
	{
		JFormattedTextField jFormatText;
		
		jFormatText = (JFormattedTextField) controls.getComponent("Roll1TextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));
		jFormatText = (JFormattedTextField) controls.getComponent("Roll2TextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));
		jFormatText = (JFormattedTextField) controls.getComponent("Roll3TextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));
		jFormatText = (JFormattedTextField) controls.getComponent("RollSTextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));

		jFormatText = (JFormattedTextField) controls.getComponent("AngleRTextBox");
		setConstraints(jFormatText, new Double(0.0), new Double(90.0));
		jFormatText = (JFormattedTextField) controls.getComponent("AngleSTextBox");
		setConstraints(jFormatText, new Double(0.0), new Double(90.0));

		jFormatText = (JFormattedTextField) controls.getComponent("RadiusSTextBox");
		setConstraints(jFormatText, new Double(0.0));
	}

	/**
	 * Set document listeners.
	 */
	private void setListeners()
	{
		JTextField jText;

		jText = (JTextField) controls.getComponent("LengthRTextBox");
		lengthListener = new LengthListener();
		jText.getDocument().addDocumentListener(lengthListener);

		jText = (JTextField) controls.getComponent("RadiusRTextBox");
		radiusListener = new RadiusListener();
		jText.getDocument().addDocumentListener(radiusListener);

		jText = (JTextField) controls.getComponent("AngleRTextBox");
		angleListener = new AngleListener();
		jText.getDocument().addDocumentListener(angleListener);

		jText = (JTextField) controls.getComponent("LengthSTextBox");
		lengthListener2 = new LengthListener2();
		jText.getDocument().addDocumentListener(lengthListener2);

		jText = (JTextField) controls.getComponent("RadiusSTextBox");
		radiusListener2 = new RadiusListener2();
		jText.getDocument().addDocumentListener(radiusListener2);

		jText = (JTextField) controls.getComponent("AngleSTextBox");
		angleListener2 = new AngleListener2();
		jText.getDocument().addDocumentListener(angleListener2);
	}

	/*
	 * ---------- Inner classes -----------
	 */
	
	abstract class DocumentAdapter implements DocumentListener
	{
		public void changedUpdate(DocumentEvent de)
		{
			act(de);
		}

		public void insertUpdate(DocumentEvent de)
		{
			act(de);
		}

		public void removeUpdate(DocumentEvent de)
		{
			act(de);
		}

		abstract protected void act(DocumentEvent de);		
	}

	class AngleListener extends DocumentAdapter
	{
		protected void act(DocumentEvent de)
		{
			JFormattedTextField jFormat;
			JFormattedTextField jRadius;
			JFormattedTextField jLength;

			jFormat = (JFormattedTextField) controls.getComponent("AngleRTextBox");
			jRadius = (JFormattedTextField) controls.getComponent("RadiusRTextBox");
			jLength = (JFormattedTextField) controls.getComponent("LengthRTextBox");

			double angle = ((Number) jFormat.getValue()).doubleValue();

			double radius = ((Number) jRadius.getValue()).doubleValue();
			double length = angle / 180.0 * (Math.PI * radius);

			jLength.getDocument().removeDocumentListener( lengthListener );
			jLength.setValue( new Double(length) );
			jLength.getDocument().addDocumentListener( lengthListener );
		}
	}

	class LengthListener extends DocumentAdapter
	{
		protected void act(DocumentEvent de)
		{
			JFormattedTextField jAngle;
			JFormattedTextField jRadius;
			JFormattedTextField jLength;

			jAngle = (JFormattedTextField) controls.getComponent("AngleRTextBox");
			jRadius = (JFormattedTextField) controls.getComponent("RadiusRTextBox");
			jLength = (JFormattedTextField) controls.getComponent("LengthRTextBox");

			double length = ((Number) jLength.getValue()).doubleValue();

			double radius = ((Number) jRadius.getValue()).doubleValue();

			if (Math.abs(radius) != 0.0)
			{
			double angle = length * 180.0 / (Math.PI * radius);

			jAngle.getDocument().removeDocumentListener(angleListener);
			jAngle.setValue( new Double(angle) );
			jAngle.getDocument().addDocumentListener(angleListener);
			}
		}
	}

	class RadiusListener extends DocumentAdapter
	{
		protected void act(DocumentEvent de)
		{
			JFormattedTextField jAngle;
			JFormattedTextField jRadius;
			JFormattedTextField jLength;

			jAngle = (JFormattedTextField) controls.getComponent("AngleRTextBox");
			jRadius = (JFormattedTextField) controls.getComponent("RadiusRTextBox");
			jLength = (JFormattedTextField) controls.getComponent("LengthRTextBox");

			double radius = ((Number) jRadius.getValue()).doubleValue();
			if (Math.abs(radius) == 0.0)
			{
				jAngle.setEnabled(false);
			}
			else
			{
				if (!jAngle.isEnabled())
				{
					jAngle.setEnabled(true);

					double length = ((Number) jLength.getValue()).doubleValue();
					double angle = length / (radius * Math.PI) * 180.0;

					jAngle.getDocument().removeDocumentListener(angleListener);
					jAngle.setValue( new Double(angle) );
					jAngle.getDocument().addDocumentListener(angleListener);
				}
				else
				{
					double angle = ((Number) jAngle.getValue()).doubleValue();
					double length =  angle / 180.0 * (Math.PI * radius);

					jLength.getDocument().removeDocumentListener(lengthListener);
					jLength.setValue( new Double(length) );
					jLength.getDocument().addDocumentListener(lengthListener);
				}
			}
		}
	}

	class AngleListener2 extends DocumentAdapter
	{
		protected void act(DocumentEvent de)
		{
			JFormattedTextField jFormat;
			JFormattedTextField jRadius;
			JFormattedTextField jLength;

			jFormat = (JFormattedTextField) controls.getComponent("AngleSTextBox");
			jRadius = (JFormattedTextField) controls.getComponent("RadiusSTextBox");
			jLength = (JFormattedTextField) controls.getComponent("LengthSTextBox");

			double angle = ((Number) jFormat.getValue()).doubleValue();
			double radius = ((Number) jRadius.getValue()).doubleValue();
			double length = angle / 180.0 * (Math.PI * radius);

			jLength.getDocument().removeDocumentListener( lengthListener2 );
			jLength.setValue( new Double(length) );
			jLength.getDocument().addDocumentListener( lengthListener2 );
		}
	}

	class LengthListener2 extends DocumentAdapter
	{
		protected void act(DocumentEvent de)
		{
			JFormattedTextField jAngle;
			JFormattedTextField jRadius;
			JFormattedTextField jLength;

			jAngle = (JFormattedTextField) controls.getComponent("AngleSTextBox");
			jRadius = (JFormattedTextField) controls.getComponent("RadiusSTextBox");
			jLength = (JFormattedTextField) controls.getComponent("LengthSTextBox");

			double length = ((Number) jLength.getValue()).doubleValue();
			double radius = ((Number) jRadius.getValue()).doubleValue();
			double angle = length * 180.0 / (Math.PI * radius);

			jAngle.getDocument().removeDocumentListener(angleListener2);
			jAngle.setValue( new Double(angle) );
			jAngle.getDocument().addDocumentListener(angleListener2);
		}
	}

	class RadiusListener2 extends DocumentAdapter
	{
		protected void act(DocumentEvent de)
		{
			JFormattedTextField jAngle;
			JFormattedTextField jRadius;
			JFormattedTextField jLength;

			jAngle = (JFormattedTextField) controls.getComponent("AngleSTextBox");
			jRadius = (JFormattedTextField) controls.getComponent("RadiusSTextBox");
			jLength = (JFormattedTextField) controls.getComponent("LengthSTextBox");

			double radius = ((Number) jRadius.getValue()).doubleValue();
			double angle = ((Number) jAngle.getValue()).doubleValue();
			double length =  angle / 180.0 * (Math.PI * radius);

			jLength.getDocument().removeDocumentListener(lengthListener2);
			jLength.setValue( new Double(length) );
			jLength.getDocument().addDocumentListener(lengthListener2);
		}
	}

	class OKAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			double d;
			int i;
			int handler = 0;
			JToggleButton		jToggle;
			JFormattedTextField jFormat;
			JTextField			jText;
			JRadioButton		jRadio;
			JComboBox			jComboBox;

			TrackFactory factory = TrackFactory.getInstance();

			jToggle = (JToggleButton) controls.getComponent("CommonHandlerButton");
			if (jToggle.isSelected())
				handler = TrackFactory.HANDLER_LEFT;

			jToggle = (JToggleButton) controls.getComponent("StraightHandlerButton");
			if (jToggle.isSelected())
				handler = TrackFactory.HANDLER_RIGHT;

			jToggle = (JToggleButton) controls.getComponent("SideHandlerButton");
			if (jToggle.isSelected())
				handler = TrackFactory.HANDLER_SIDE;

			jFormat = (JFormattedTextField) controls.getComponent("LengthRTextBox");
			double lenR = ((Number) jFormat.getValue()).doubleValue();

			jFormat = (JFormattedTextField) controls.getComponent("RadiusRTextBox");
			double radR = ((Number) jFormat.getValue()).doubleValue();

			jFormat = (JFormattedTextField) controls.getComponent("LengthSTextBox");
			double lenS = ((Number) jFormat.getValue()).doubleValue();

			jFormat = (JFormattedTextField) controls.getComponent("RadiusSTextBox");
			double radS = ((Number) jFormat.getValue()).doubleValue();

			jRadio = (JRadioButton) controls.getComponent("LeftRadioButton");
			if (jRadio.isSelected())
				radS = -radS;

			Switch sswitch = factory.createSwitch(handler, lenR, radR, lenS, radS);

			// Name
			jText = (JTextField) controls.getComponent("NameTextBox");
			String name = jText.getText();
			if (name.equals(""))
				name = null;
			sswitch.setName( name );

			// Width
			jFormat = (JFormattedTextField) controls.getComponent("WidthTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			sswitch.setWidth( d );

			// Tap
			jFormat = (JFormattedTextField) controls.getComponent("TapTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			sswitch.setTap( d );

			// Friction
			jFormat = (JFormattedTextField) controls.getComponent("FrictionTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			sswitch.setFriction( d );

			// Damage
			jFormat = (JFormattedTextField) controls.getComponent("DamageTextBox");
			i = ((Number) jFormat.getValue()).intValue();
			sswitch.setDamage( i );

			// Quality
			jFormat = (JFormattedTextField) controls.getComponent("QualityTextBox");
			i = ((Number) jFormat.getValue()).intValue();
			sswitch.setQuality( i );

			// Enviroment
			jComboBox = (JComboBox) controls.getComponent("EnviromentComboBox");
			sswitch.setEnviroment( (String) jComboBox.getSelectedItem() );

			// Visibility
			jRadio = (JRadioButton) controls.getComponent("VisibleRadioButton");
			sswitch.setVisible( jRadio.isSelected() );

			// Texture
			jComboBox = (JComboBox) controls.getComponent("TextureComboBox");
			sswitch.setTexture( (String) jComboBox.getSelectedItem() );

			// Second texture
			jComboBox = (JComboBox) controls.getComponent("SecondTextureComboBox");
			sswitch.setSecondTexture( (String) jComboBox.getSelectedItem() );

			// Texture length
			jFormat = (JFormattedTextField) controls.getComponent("TextureLengthEditBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			sswitch.setTextureLength(d);

			// Texture width
			jFormat = (JFormattedTextField) controls.getComponent("TextureWidthTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			sswitch.setTextureWidth(d);

			// Texture height
			jFormat = (JFormattedTextField) controls.getComponent("TextureHeightTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			sswitch.setTextureHeight(d);

			// Texture slope
			jFormat = (JFormattedTextField) controls.getComponent("TextureSlopeTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			sswitch.setTextureSlope(d);

			// Roll 1
			jFormat = (JFormattedTextField) controls.getComponent("Roll1TextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			sswitch.setRoll1(d);

			// Roll 2
			jFormat = (JFormattedTextField) controls.getComponent("Roll2TextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			sswitch.setRoll2(d);

			// Roll switched
			jFormat = (JFormattedTextField) controls.getComponent("RollSTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			sswitch.setSwitchedRoll(d);

			// Roll 3
			jFormat = (JFormattedTextField) controls.getComponent("Roll3TextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			sswitch.setRoll3(d);

			// Wsad to teraz do scenerii
			SceneryFrame frame = application.getTopmostFrame();
			if (frame != null)
			{
				JSceneryEditor editor = frame.getSceneryEditor();

				BasePoint basePoint = editor.getBasePoint();
				
				if (basePoint != null)
				{
				// Wykonaj obrt toru
				double alpha = - basePoint.getAlpha();
				double beta = - basePoint.getBeta();

				Matrix4d rotMatrix = new Matrix4d();
				rotMatrix.rotY(alpha);

				rotMatrix.transform( sswitch.getPoint1() );
				rotMatrix.transform( sswitch.getVector1() );
				rotMatrix.transform( sswitch.getVector2() );
				rotMatrix.transform( sswitch.getPoint2() );
				rotMatrix.transform( sswitch.getPoint3() );
				rotMatrix.transform( sswitch.getVector3() );
				rotMatrix.transform( sswitch.getSwitchedVector() );

				AxisAngle4d rotAxis = 
					new AxisAngle4d(sswitch.getPoint2().z, 0.0, -sswitch.getPoint2().x, beta);
				rotMatrix = new Matrix4d();
				rotMatrix.set(rotAxis);

				rotMatrix.transform( sswitch.getPoint1() );
				rotMatrix.transform( sswitch.getVector1() );
				rotMatrix.transform( sswitch.getVector2() );
				rotMatrix.transform( sswitch.getPoint2() );
				rotMatrix.transform( sswitch.getPoint3() );
				rotMatrix.transform( sswitch.getVector3() );
				rotMatrix.transform( sswitch.getSwitchedVector() );

				// wykonaj przesunicie toru
				Matrix4d traMatrix = new Matrix4d();
				traMatrix.setIdentity();
				traMatrix.setTranslation( new Vector3d(basePoint.getPoint()) );

				traMatrix.transform( sswitch.getPoint1() );
				traMatrix.transform( sswitch.getVector1() );
				traMatrix.transform( sswitch.getVector2() );
				traMatrix.transform( sswitch.getPoint2() );
				traMatrix.transform( sswitch.getPoint3() );
				traMatrix.transform( sswitch.getVector3() );
				traMatrix.transform( sswitch.getSwitchedVector() );

				Scenery scenery = editor.getScenery();
				Object[] obj = new Object[1];
				obj[0] = sswitch;
				scenery.insert(obj);

				// Zaznacz nowy obiekt
				editor.setSelectedObject(sswitch);

				// Ustaw nowy punkt bazowy
				Point3d p1 = sswitch.getPoint1();
				Point3d p2 = sswitch.getPoint2();
				Point3d p3 = sswitch.getPoint3();
				Point3d newPoint;
				Vector3d resolved = null;

				if (basePoint.getPoint().epsilonEquals(p1, 0.001))
					newPoint = new Point3d(p2);
				else 
					newPoint = new Point3d(p1);
				
				if (newPoint.equals(p1))
				{
					if (!sswitch.isStraight())
					{
						resolved = new Vector3d( sswitch.getVector1() );
						resolved.negate();
					}
					else
					{
						Point3d p = p2;
						resolved = new Vector3d();
						resolved.x = newPoint.x - p.x;
						resolved.y = newPoint.y - p.y;
						resolved.z = newPoint.z - p.z;
					}
				}
				else if (newPoint.equals(p2))
				{
					if (!sswitch.isStraight())
					{
						resolved = new Vector3d( sswitch.getVector2() );
						resolved.negate();
					}
					else
					{
						Point3d p = p1;
						resolved = new Vector3d();
						resolved.x = newPoint.x - p.x;
						resolved.y = newPoint.y - p.y;
						resolved.z = newPoint.z - p.z;
					}
				}
				else if (newPoint.equals(p3))
				{
					resolved = new Vector3d( sswitch.getVector3() );
					resolved.negate();
				}
					
				BasePoint newBasePoint = new BasePoint(newPoint, resolved);
				editor.setBasePoint(newBasePoint);
				}
			}

			saveConfiguration(sswitch);
			dispose();
		}

		private void saveConfiguration(Switch sswitch)
		{
			Configuration config = application.config;

			config.put("CustomSwitchProperties.length1", new Double(sswitch.getLength()));
			config.put("CustomSwitchProperties.radius1", new Double(Math.abs(sswitch.getRadius())));
			config.put("CustomSwitchProperties.radius2", new Double(Math.abs(sswitch.getSwitchedRadius())));
			config.put("CustomSwitchProperties.width", new Double(sswitch.getWidth()));
			config.put("CustomSwitchProperties.friction", new Double(sswitch.getFriction()));
			config.put("CustomSwitchProperties.tap", new Double(sswitch.getTap()));
			config.put("CustomSwitchProperties.damage", new Integer(sswitch.getDamage()));
			config.put("CustomSwitchProperties.quality", new Integer(sswitch.getQuality()));
			config.put("CustomSwitchProperties.enviroment", sswitch.getEnviroment());
			config.put("CustomSwitchProperties.visibility", new Boolean(sswitch.isVisible()));
			config.put("CustomSwitchProperties.texture", sswitch.getTexture());
			config.put("CustomSwitchProperties.texLength", new Double(sswitch.getTextureLength()));
			config.put("CustomSwitchProperties.secondTexture", sswitch.getSecondTexture());
			config.put("CustomSwitchProperties.texWidth", new Double(sswitch.getTextureWidth()));
			config.put("CustomSwitchProperties.texHeight", new Double(sswitch.getTextureHeight()));
			config.put("CustomSwitchProperties.texSlope", new Double(sswitch.getTextureSlope()));
			config.put("CustomSwitchProperties.roll1", new Double(sswitch.getRoll1()));
			config.put("CustomSwitchProperties.roll2", new Double(sswitch.getRoll2()));
			config.put("CustomSwitchProperties.rollS", new Double(sswitch.getSwitchedRoll()));
			config.put("CustomSwitchProperties.roll3", new Double(sswitch.getRoll3()));
			
			JFormattedTextField jFormat;
			jFormat = (JFormattedTextField) controls.getComponent("LengthSTextBox");
			double d = ((Number) jFormat.getValue()).doubleValue();
			config.put("CustomSwitchProperties.length2", new Double(d));

			JRadioButton jRadio;
			jRadio = (JRadioButton) controls.getComponent("RightRadioButton");
			config.put("CustomSwitchProperties.right", new Boolean(jRadio.isSelected()));

			JToggleButton jToggle;
			jToggle = (JToggleButton) controls.getComponent("CommonHandlerButton");
			if (jToggle.isSelected())
				config.put(
					"CustomSwitchProperties.handler", 
					new Integer(TrackFactory.HANDLER_LEFT)
				);

			jToggle = (JToggleButton) controls.getComponent("StraightHandlerButton");
			if (jToggle.isSelected())
				config.put(
					"CustomSwitchProperties.handler", 
					new Integer(TrackFactory.HANDLER_RIGHT)
				);

			jToggle = (JToggleButton) controls.getComponent("SideHandlerButton");
			if (jToggle.isSelected())
				config.put(
					"CustomSwitchProperties.handler",
					new Integer(TrackFactory.HANDLER_SIDE)
				);

		}
	}

	/*
	 * ------------- Members -------------
	 */
	
	/** Application. */
	private Jet application;

	/** Listeners. */
	private DocumentListener angleListener;
	private DocumentListener lengthListener;
	private DocumentListener radiusListener;

	private DocumentListener angleListener2;
	private DocumentListener lengthListener2;
	private DocumentListener radiusListener2;
}
